﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.Threading;                                 // for Sleep
using FTD2XX_NET;                                       // FTD2XX Wrapper DLL for C#

namespace SDR_test1
{
    public partial class Form1 : Form
    {
        //　クラスForm2からサブフォームを2個(wave1, wave2)生成
        Form2 wave1;
        Form2 wave2;

        public bool True { get; private set; }

        //-------------------------------------------------------------------- グローバル変数クラスの定義 ----------------------------------
        public static class Global
        {
            public static double[] ave = new double[512];
            public static double sumave;
        }

        //　メインのForm1とサブフォーム2個を表示
        public Form1()
        {
            InitializeComponent();

            //　サブフォーム Wave1 表示
            wave1 = new Form2();
            wave1.Show();

            //　サブフォーム Wave2 表示
            wave2 = new Form2();
            wave2.Show();
        }

        //　ボタン Start
        private void button1_Click(object sender, EventArgs e)
        {
            timer1.Enabled = true;
        }

        //　ボタン Stop
        private void button2_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
        }

        //-------------------------------------------------------------------- FT232HLコマンド生成 -----------------------------------------
        private static byte[] FT232(ushort adrs)                                // FT232HL command generator
        {
            byte b1 = (byte)((adrs & 0xFF00) / 0x0100);                         // Command (bit 15-12) and Address (bit 11- 8)
            byte b2 = (byte)((adrs & 0x00FF) / 0x0001);                         // Address (bit  7- 0)

            byte[] buf = {  0x80, 0x00, 0x0B,                                   // ADBUS port command, CS -> L, port direction
                            0x86, 0x00, 0x00,                                   // Clock divider 60MHz/((1+0x00*256+0x00)*2)=30MHz
                            //0x86, 0x01, 0x00,                                 // Clock divider 60MHz/((1+0x00*256+0x01)*2)=15MHz
                            0x11, 0x01, 0x00,                                   // SPI port command TX, 2byte(0x01 0x00)
                            b1, b2,                                             // Command and Address 
                            0x20, 0x01, 0x00, 0x87,                             // SPI port command RX, 2byte(0x01 0x00), Send immediate
                            0x80, 0x08, 0x0B,                                   // ADBUS port command, CS -> H, port direction
                         };

            return buf;
        }

        //-------------------------------------------------------------------- FT232HL波形キャプチャ ---------------------------------------
        private new void Capture(double[] wfm)
        {
            byte[] wave = new byte[2048];         
            byte[] wbuf;

            uint writtenLength = 0;
            uint readLength = 0;
            uint RxLength = 0;
            ushort adrs = 0x0000;

            //-------------------------------------------------------------------- FTDI 通信オープン ---------------------------------------
            FTDI ftdev = new FTDI();                                                    // Device Driver
            ftdev.OpenByIndex(0);                                                       // Open Device Index=0
            ftdev.SetBitMode(0, 2);                                                     // Mask=0, Mode=2(MPSSE)

            //-------------------------------------------------------------------- キャプチャ　リセット ------------------------------------
            wbuf = FT232(0x0000);                                                       // Get Command
            ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command

            //-------------------------------------------------------------------- キャプチャ　スタート ------------------------------------
            if (radioButton4.Checked == true) wbuf = FT232(0x4000);                     // A/D
            if (radioButton11.Checked == true) wbuf = FT232(0x5000);                    // AWG
            if (radioButton12.Checked == true) wbuf = FT232(0x6000);                    // MIX
            if (radioButton13.Checked == true) wbuf = FT232(0x7000);                    // FIR
 
            ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command
            Thread.Sleep(5);                                                            // Wait Capture done (5ms)

            //-------------------------------------------------------------------- FTDI バッファを空にする ---------------------------------
            ftdev.GetRxBytesAvailable(ref RxLength);                                    // Get number of Garbage
            ftdev.Read(wave, RxLength, ref readLength);                                 // Read Data

            //-------------------------------------------------------------------- RAM 読出コマンド ----------------------------------------
            for (adrs = 0x1000; adrs < 0x1200; adrs++)                                  // RAM read 0x000 to 0x200 (0 to 511) CH1
            {
                wbuf = FT232(adrs);                                                     // Get Command
                ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                      // Write Command
            }

            //-------------------------------------------------------------------- FTDI バッファが一杯になったら読み出す -------------------
            int tout = 0;
            do
            {
                ftdev.GetRxBytesAvailable(ref RxLength);                                // Wait for return data
                tout++;                                                                 // Timeout count                                                        
            } while (RxLength < 1023 & tout < 1023);                                    // Length = RAM data x2

            ftdev.Read(wave, RxLength, ref readLength);                                 // Read Data

            //-------------------------------------------------------------------- FTDI 通信クローズ ---------------------------------------
            ftdev.Close();     
                                
            //-------------------------------------------------------------------- 2byte SPIデータをfloatに変換 ----------------------------
            for (int n = 0; n < 512; n++)
            {
                byte b1 = wave[2 * n + 0];
                byte b2 = wave[2 * n + 1];
                int b0 = (b1 & 0x3F) * 0x0100 + b2;                 // 14bit <= 6bit + 8bit
                if (radioButton4.Checked == true  ||                // A/D
                    radioButton11.Checked == true ||                // AWG
                    radioButton12.Checked == true ||                // MIX
                    radioButton13.Checked == true )                 // FIR
                {
                    if (b0 < 8192) wfm[n] = (float)(b0 / 8191.0);                                       // Pos
                    if (b0 > 8191) wfm[n] = (float)((b0 - 16384.0) / 8191.0);                           // Neg
                }
                if (radioButton5.Checked == true) wfm[n] = (float)Math.Sin(Math.PI * n / 8);            // Sin(pi)
                if (radioButton6.Checked == true) wfm[n] = (float)Math.Sin(3.14 * n / 8);               // Sin(3.14)
                if (radioButton7.Checked == true) wfm[n] = 1.0;                                         // DC=1.0
                if (radioButton10.Checked == true) wfm[n] = (float)Math.Sin(Math.PI * n * n / 1024);    // Charp 
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            double[] Ich = new double[1024];
            double[] Qch = new double[1024];
            double[] fft = new double[1024];
            double[] pix = new double[1024];
            double sum;

            //　平均値の長さを得る
            int acnt = (int)numericUpDown1.Value;

            //　パワー測定範囲を得る
            int bwstart = (int)numericUpDown2.Value;
            int bwstop  = (int)numericUpDown3.Value;

            //　データキャプチャ
            Ich[512] = 0;
            Ich[513] = 511;
            Capture(Ich);

            //　サブフォームに波形データを送る
            wave1.SendData = Ich;

            //　FFT Size=512 
            if (radioButton1.Checked == true) FFT(Ich, Qch, 512, 0);    // Hanning Line
            if (radioButton8.Checked == true) FFT(Ich, Qch, 512, 1);    // Hanning Power
            if (radioButton2.Checked == true) FFT(Ich, Qch, 512, 2);    // Blackman line
            if (radioButton9.Checked == true) FFT(Ich, Qch, 512, 3);    // Blackman Power
            if (radioButton3.Checked == true) FFT(Ich, Qch, 512, 4);    // Square

            //  -1.0～1.0の範囲の真数をlog10(dB)に変換
            sum = xdb(Ich, Qch, fft, 256, bwstart, bwstop);

            //　平均値処理
            average(fft, pix, sum, 512, acnt);

            //　積分範囲を渡す
            pix[512] = 2 * bwstart;
            pix[513] = 2 * bwstop;

            //　サブフォームに波形データを送る
            wave2.SendData = pix;
        }

        //　平均値処理
        private void average(double[] avein, double[] aveout, double sum, int size, int acount)
        {
            double vpeak = -100;
            double fpeak = 0;
            double gain = 1.0 / acount;

            //　パワー測定範囲を得る
            int bwstart = (int)numericUpDown2.Value;
            int bwstop = (int)numericUpDown3.Value;

            //　平均値処理とピーク検出
            for (int n = 0; n < size; n++)
            {
                //　平均値
                Global.ave[n] = (1 - gain) * Global.ave[n] + gain * avein[n];
                aveout[n] = 0.2 * Global.ave[n] / 10 + 1;

                //　ピーク検出
                if (vpeak < Global.ave[n])
                {
                    vpeak = Global.ave[n];   
                    fpeak = n;
                }
            }

            //　全帯域電力の平均値
            Global.sumave = (1 - gain) * Global.sumave + gain * sum;

            //　全帯域の電力
            double pa = 10 * Math.Log10(Global.sumave);
            double pb = 10 * Math.Log10(Global.sumave / (bwstop - bwstart));

            //　テキスト表示
            textBox1.Text = (Math.Round(vpeak * 1000) / 1000).ToString();
            textBox2.Text = (125 * fpeak / 512).ToString();
            textBox3.Text = (Math.Round(pa * 1000) / 1000).ToString();
            textBox4.Text = (Math.Round(pb * 1000) / 1000).ToString();

        }

        //  真数をlog10(dB)に変換
        private static double xdb(double[] Ich, double[] Qch, double[] vdb, int size, int bwstart, int bwstop)
        {
            double sum = 0;

            for (int n = 0; n < size; n++)
            {
                //　１に正規化
                double i = Ich[n] / 512;
                double q = Qch[n] / 512;

                //　二乗加算平方根
                double vr = Math.Sqrt(i * i + q * q);

                //　指定帯域を積算
                if (n >= bwstart && n <= bwstop) sum = sum + vr * vr;
                if (bwstart > bwstop) sum = 0;

                //　±1をdBに変換
                double spc = (double)(20 * Math.Log10(vr));                  

                //　-100dBで制限する
                if (spc < -100) spc = -100;                   
                
                //　倍の長さで結果を出力
                vdb[2 * n    ] = spc;
                vdb[2 * n + 1] = spc;
            }

            return sum;
        }

        //  FFT
        private void FFT(double[] real, double[] imag, int length, int sw)
        {
            int i, j, k, l, half;
            double m, h, g, q, p, phase;
            double a = 0;
            double b = 0;
            double swap;

            half = length / 2;

            //------------------------------- Window --------------------------------------
            for (i = 0; i < length; i++)
            {

                phase = 2 * System.Math.PI * (float)i / (float)length;                  // 0 to 2*pi

                if (sw == 0)        // Hanning Line
                {
                    a = 2.00000 * real[i] * (0.50 - 0.50 * System.Math.Cos(phase));
                    b = 2.00000 * imag[i] * (0.50 - 0.50 * System.Math.Cos(phase));
                }
                if (sw == 1)        // Hanning Power
                {
                    a = 1.63299 * real[i] * (0.50 - 0.50 * System.Math.Cos(phase));
                    b = 1.63299 * imag[i] * (0.50 - 0.50 * System.Math.Cos(phase));
                }
                if (sw == 2)        // Blackman Line
                {
                    a = 2.78746 * real[i] * (0.35875 - 0.48829 * System.Math.Cos(phase) + 0.14128 * System.Math.Cos(2 * phase) - 0.01168 * System.Math.Cos(3 * phase)); 
                    b = 2.78746 * imag[i] * (0.35875 - 0.48829 * System.Math.Cos(phase) + 0.14128 * System.Math.Cos(2 * phase) - 0.01168 * System.Math.Cos(3 * phase));
                }
                if (sw == 3)        // Blackman Power
                {
                    a = 1.96870 * real[i] * (0.35875 - 0.48829 * System.Math.Cos(phase) + 0.14128 * System.Math.Cos(2 * phase) - 0.01168 * System.Math.Cos(3 * phase));
                    b = 1.96870 * imag[i] * (0.35875 - 0.48829 * System.Math.Cos(phase) + 0.14128 * System.Math.Cos(2 * phase) - 0.01168 * System.Math.Cos(3 * phase)); 
                }
                if (sw == 4)        // Square
                {
                    a = 1.00000 * real[i];
                    b = 1.00000 * imag[i];
                }

                real[i] = a;
                imag[i] = b;
            }

            //--------------------------------- FFT --------------------------------------
            m = System.Math.Round(System.Math.Log(length) / System.Math.Log(2));

            l = length;
            h = 1;

            for (g = 1; g <= m; g++)
            {
                l = l / 2;
                k = 0;
                for (q = 1; q <= h; q++)
                {
                    p = 0;
                    for (i = k; i <= l + k - 1; i++)
                    {
                        j = i + l;
                        a = real[i] - real[j];
                        b = imag[i] - imag[j];
                        real[i] = real[i] + real[j];
                        imag[i] = imag[i] + imag[j];
                        if (p == 0)
                        {
                            real[j] = a;
                            imag[j] = b;
                        }
                        else
                        {
                            real[j] = a * System.Math.Cos(2 * System.Math.PI * p / (float)length) + b * System.Math.Sin(2 * System.Math.PI * p / (float)length);
                            imag[j] = b * System.Math.Cos(2 * System.Math.PI * p / (float)length) - a * System.Math.Sin(2 * System.Math.PI * p / (float)length);
                        }
                        p = p + h;
                    }
                    k = k + l + l;
                }
                h = h + h;
            }

            j = length / 2;

            for (i = 1; i <= length - 1; i++)
            {
                k = length;
                if (j < i)
                {
                    swap = real[i];
                    real[i] = real[j];
                    real[j] = swap;
                    swap = imag[i];
                    imag[i] = imag[j];
                    imag[j] = swap;
                }

                do
                {
                    k = k / 2;
                    j = j - k;
                    if (k == 0 && j == 0) break;
                } while ((j + k) >= k);

                j = j + k + k;
            }

        }

        private void radioButton8_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void radioButton10_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void groupBox3_Enter(object sender, EventArgs e)
        {

        }

        private void button3_Click(object sender, EventArgs e)
        {
            byte[] wbuf;

            uint writtenLength = 0;
            int freq = (int)numericUpDown4.Value;
            int ampl = (int)numericUpDown5.Value;

            //-------------------------------------------------------------------- 波形周期計算 --------------------------------------------
            float ts = 1000000 / (float)freq;                                           // 1 cycle in us
            float smp = ts / 1;                                                         // 1 cycle in Sample
            int mag = (int)(4095 / smp);                                                // Get Magnification
            int leng = (int)Math.Round(mag * smp, 0);                                   // Get Length
            float fgen = 1000000 * mag / ((float)leng * 1);                             // Get Generated Freq
            textBox5.Text = fgen.ToString();                                            // Textout Generated Freq
            textBox6.Text = leng.ToString();                                            // Textout Generated Length

            //-------------------------------------------------------------------- FTDI 通信オープン ---------------------------------------
            FTDI ftdev = new FTDI();                                                    // Device Driver
            ftdev.OpenByIndex(0);                                                       // Open Device Index=0
            ftdev.SetBitMode(0, 2);                                                     // Mask=0, Mode=2(MPSSE)

            //-------------------------------------------------------------------- 波形長さ転送 --------------------------------------------
            wbuf = FT232((ushort)(0x3000 + leng));                                      // Write AWG + Data 12bit
            ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command

            //-------------------------------------------------------------------- データ転送 ----------------------------------------------
            for (int n = 0; n < leng; n++)
            {
                int dat = (int)(ampl * Math.Sin(mag * 2.0 * Math.PI * n / leng));       // Sin Wave
                if (dat < 0) dat = dat + 4095;                                          // Transrate to comp by 2
                wbuf = FT232((ushort)(0x2000 + dat));                                   // Generate Data
                ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                      // Write Command
            }

            //-------------------------------------------------------------------- 波形長さ転送 --------------------------------------------
            wbuf = FT232((ushort)(0x3000 + leng));                                      // Write AWG + Data 12bit
            ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command

            //-------------------------------------------------------------------- FTDI 通信クローズ ---------------------------------------
            ftdev.Close();
        }

        private void label10_Click(object sender, EventArgs e)
        {

        }

        private void numericUpDown4_ValueChanged(object sender, EventArgs e)
        {

        }

        private void button4_Click(object sender, EventArgs e)
        {
            byte[] wbuf;

            uint writtenLength = 0;
            int freq = (int)numericUpDown4.Value;
            int ampl = (int)numericUpDown5.Value;

            //-------------------------------------------------------------------- 波形周期計算 --------------------------------------------
            float ts = 1000000 / (float)freq;                                           // 1 cycle in us
            float smp = ts / 1;                                                         // 1 cycle in Sample
            int mag = (int)(4095 / smp);                                                // Get Magnification
            int leng = (int)Math.Round(mag * smp, 0);                                   // Get Length
            float fgen = 1000000 * mag / ((float)leng * 1);                             // Get Generated Freq
            textBox5.Text = fgen.ToString();                                            // Textout Generated Freq
            textBox6.Text = leng.ToString();                                            // Textout Generated Length

            //-------------------------------------------------------------------- FTDI 通信オープン ---------------------------------------
            FTDI ftdev = new FTDI();                                                    // Device Driver
            ftdev.OpenByIndex(0);                                                       // Open Device Index=0
            ftdev.SetBitMode(0, 2);                                                     // Mask=0, Mode=2(MPSSE)

            //-------------------------------------------------------------------- 波形長さ転送 --------------------------------------------
            wbuf = FT232((ushort)(0x3000 + leng));                                      // Write AWG + Data 12bit
            ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command

            //-------------------------------------------------------------------- データ転送 ----------------------------------------------
            for (int n = 0; n < leng; n++)
            {
                int dat = (int)(100000 * Math.Sin(mag * 2.0 * Math.PI * n / leng));     // Sin Wave
                if (dat > ampl) dat = ampl;                                             // Limitter +
                if (dat < -ampl) dat = -ampl;                                           // Limitter -
                if (dat < 0) dat = dat + 4095;                                          // Transrate to comp by 2
                wbuf = FT232((ushort)(0x2000 + dat));                                   // Generate Data
                ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                      // Write Command
            }

            //-------------------------------------------------------------------- 波形長さ転送 --------------------------------------------
            wbuf = FT232((ushort)(0x3000 + leng));                                      // Write AWG + Data 12bit
            ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command

            //-------------------------------------------------------------------- FTDI 通信クローズ ---------------------------------------
            ftdev.Close();
        }

        private void radioButton11_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void radioButton12_CheckedChanged(object sender, EventArgs e)
        {

        }
    }
}